Package: eva3dm
Type: Package
Title: Evaluation of 3D Meteorological and Air Quality Models
Version: 1.20
Date: 2026-04-21
Description: Provides tools for post-process, evaluate and visualize results from 3d Meteorological and Air Quality models against point observations (i.e. surface stations) and grid (i.e. satellite) observations.
Authors@R: c(
  person(given = "Daniel",
         family = "Schuch",
         role = c("aut", "cre"),
         email = "underschuch@gmail.com",
         comment = c(ORCID = "0000-0001-5977-4519"))
         )
Maintainer: Daniel Schuch <underschuch@gmail.com>
License: MIT + file LICENSE
Imports: terra, ncdf4, utils
Suggests: knitr, riem, rmarkdown, testthat (>= 3.0.0)
Encoding: UTF-8
BugReports: https://github.com/Schuch666/eva3dm/issues/
RoxygenNote: 7.3.2
Depends: R (>= 3.5.0)
URL: https://schuch666.github.io/eva3dm/
Config/testthat/edition: 3
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2026-04-22 01:51:39 UTC; schuch
Author: Daniel Schuch [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-5977-4519>)
Repository: CRAN
Date/Publication: 2026-04-22 03:30:02 UTC
Built: R 4.6.0; ; 2026-04-25 18:43:34 UTC; unix
