# Generated by roxygen2: do not edit by hand

export("%IN%")
export("%at%")
export(atr)
export(calculate_column)
export(cate)
export(daily)
export(eva)
export(extract_max_8h)
export(extract_mean)
export(extract_serie)
export(extract_surgical)
export(get_distances)
export(hourly)
export(interp)
export(legend_range)
export(ma8h)
export(mda8)
export(monthly)
export(ncdump)
export(overlay)
export(plot_diff)
export(plot_rast)
export(q2rh)
export(rain)
export(rast_to_netcdf)
export(read_stat)
export(rh2q)
export(sat)
export(select)
export(stat)
export(template)
export(uv2wd)
export(uv2ws)
export(vars)
export(wrf_rast)
export(wrf_sds)
export(write_stat)
export(yearly)
import(ncdf4)
import(terra)
importFrom(stats,as.formula)
importFrom(utils,menu)
importFrom(utils,read.csv)
importFrom(utils,read.table)
importFrom(utils,write.csv)
importFrom(utils,write.table)
